import { fileURLToPath } from 'url';
import { defu } from 'defu';
import { defineNuxtModule, createResolver, addImports, addComponent, addServerHandler } from '@nuxt/kit';

const DEFAULT_CACHE_TAG_INVALIDATION_DELAY = 6e4;
const DEFAULT_API_PREFIX = "/__nuxt_multi_cache";
const DEFAULT_CDN_CONTROL_HEADER = "Surrogate-Control";
const DEFAULT_CDN_TAG_HEADER = "Cache-Tag";
const defaultOptions = {
  api: {
    prefix: DEFAULT_API_PREFIX,
    cacheTagInvalidationDelay: DEFAULT_CACHE_TAG_INVALIDATION_DELAY
  }
};

const module = defineNuxtModule({
  meta: {
    name: "nuxt-multi-cache",
    configKey: "multiCache",
    version: "2.0.1",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: defaultOptions,
  async setup(passedOptions, nuxt) {
    const options = defu({}, passedOptions, {});
    const { resolve } = createResolver(import.meta.url);
    const rootDir = nuxt.options.rootDir;
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir);
    nuxt.options.runtimeConfig.multiCache = {
      rootDir,
      cdn: {
        cacheControlHeader: options.cdn?.cacheControlHeader || DEFAULT_CDN_CONTROL_HEADER,
        cacheTagHeader: options.cdn?.cacheTagHeader || DEFAULT_CDN_TAG_HEADER
      }
    };
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.externals = defu(
        typeof nitroConfig.externals === "object" ? nitroConfig.externals : {},
        {
          inline: [resolve("./runtime")]
        }
      );
    });
    if (options.data) {
      addImports({
        from: resolve("./runtime/composables/useDataCache"),
        name: "useDataCache"
      });
    }
    if (options.route) {
      addImports({
        from: resolve("./runtime/composables/useRouteCache"),
        name: "useRouteCache"
      });
    }
    if (options.cdn) {
      addImports({
        from: resolve("./runtime/composables/useCDNHeaders"),
        name: "useCDNHeaders"
      });
    }
    nuxt.options.alias["#nuxt-multi-cache/composables"] = resolve(
      "runtime/composables/index"
    );
    if (options.component) {
      await addComponent({
        filePath: resolve("./runtime/components/RenderCacheable/index"),
        name: "RenderCacheable",
        global: true
      });
    }
    if (options.component || options.route || options.data) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/cacheContext"),
        middleware: true
      });
    }
    if (options.cdn?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/cdnHeaders"),
        middleware: true
      });
    }
    if (options.route?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/serveCachedRoute")
      });
    }
    if (options.cdn?.enabled || options.route?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/responseSend")
      });
    }
    if (options.api?.enabled) {
      const apiPrefix = options.api.prefix;
      const prefix = (path) => apiPrefix + "/" + path;
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeAll"),
        method: "post",
        route: prefix("purge/all")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeTags"),
        method: "post",
        route: prefix("purge/tags")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeItem"),
        method: "post",
        route: prefix("purge/:cacheName")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/stats"),
        method: "get",
        route: prefix("stats/:cacheName")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/inspectItem"),
        method: "get",
        route: prefix("inspect/:cacheName")
      });
    }
  }
});

export { module as default };
