import { readBody, defineEventHandler, createError } from "h3";
import { getMultiCacheContext } from "./../../helpers/server.mjs";
import { DEFAULT_CACHE_TAG_INVALIDATION_DELAY } from "./../../settings.mjs";
import { getModuleConfig } from "./../helpers/index.mjs";
import { checkAuth } from "./helpers/index.mjs";
async function getTagsToPurge(event) {
  const body = await readBody(event);
  if (body && Array.isArray(body)) {
    return body;
  }
  throw createError({
    statusCode: 400,
    statusMessage: "No valid tags provided."
  });
}
function onlyUnique(value, index, self) {
  return self.indexOf(value) === index;
}
export class DebouncedInvalidator {
  constructor() {
    this.tags = [];
    this.timeout = null;
    this.delay = DEFAULT_CACHE_TAG_INVALIDATION_DELAY;
  }
  setDelay(delay = DEFAULT_CACHE_TAG_INVALIDATION_DELAY) {
    this.delay = delay;
  }
  add(tags = []) {
    tags.forEach((tag) => {
      if (!this.tags.includes(tag)) {
        this.tags.push(tag);
      }
    });
    if (!this.timeout) {
      this.timeout = setTimeout(() => {
        this.invalidate();
      }, this.delay);
    }
  }
  async invalidate() {
    if (!this.cacheContext) {
      return;
    }
    const tags = this.tags.filter(onlyUnique);
    this.tags = [];
    this.timeout = null;
    let key;
    for (key in this.cacheContext) {
      const cache = this.cacheContext[key];
      if (cache) {
        const cacheItemKeys = await cache.getKeys();
        for (const cacheKey of cacheItemKeys) {
          const item = await cache.getItem(cacheKey);
          if (item && typeof item === "object" && "cacheTags" in item) {
            const itemCacheTags = item.cacheTags;
            const shouldPurge = itemCacheTags.some((v) => {
              return tags.includes(v);
            });
            if (shouldPurge) {
              await cache.removeItem(cacheKey);
            }
          }
        }
      }
    }
    return true;
  }
}
const invalidator = new DebouncedInvalidator();
export default defineEventHandler(async (event) => {
  await checkAuth(event);
  const tags = await getTagsToPurge(event);
  if (!invalidator.cacheContext) {
    invalidator.cacheContext = getMultiCacheContext(event);
    const moduleConfig = await getModuleConfig();
    invalidator.setDelay(moduleConfig.api?.cacheTagInvalidationDelay);
  }
  invalidator.add(tags);
  return {
    status: "OK",
    tags
  };
});
